==> ctpp2-2.0.4.tar.gz: ctpp2-2.0.4/tests/VMHelloWorldGenerator.cpp <==
/*-
 * Copyright (c) 2006, 2007 FTS Team
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 4. Neither the name of the FTS Team nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      VMHelloWorldGenerator.cpp
 *
 * $FTS$
 */
#include <VMDumper.hpp>
#include <VMExecutable.hpp>
#include <VMOpcodes.h>
#include <VMInstruction.hpp>

#include <stdio.h>
#include <sysexits.h>

using namespace CTPP;

int main(int argc, char ** argv)
{
	if (argc != 2) { fprintf(stderr, "usage: %s file.name\n", argv[0]); return EX_USAGE; }

	// 1. Create static text segment
	StaticText oStaticText;

	StaticText oSyscalls;

	// Syscalls
	//                     12345678901234
	oSyscalls.StoreData("__ctpp2_output",          14); // returns '0'
	//                     1234567890123456789
	oSyscalls.StoreData("__ctpp2_output_text",     19); // returns '1'

	// Text Data
	//                     1234567890123456789
	oStaticText.StoreData("CTPP2 Hello, World\n",    19); // returns '0'
	//                     1234567890123
	oStaticText.StoreData("Concatenation",           13); // returns '1'
	//                     1
	oStaticText.StoreData(" ",                       1);  // returns '2'
	//                     1234
	oStaticText.StoreData("test.\n",                 6);  // returns '3'
	//                     1234
	oStaticText.StoreData("key",                     3);  // returns '4'

	// Numberic Data
	StaticData oStaticData;
	oStaticData.StoreInt(0);
	oStaticData.StoreInt(1977);
	oStaticData.StoreInt(10);
	oStaticData.StoreInt(8);

	// Code
	VMInstruction aCode[] =
	{
		// COMMAND              ARG RESERVERD
		// Call procedure on offset 17 from start of program
		{CALL              ,    17, 0}, // 0

		// Move integer value to top of stack
		{PUSH | ARG_SRC_STR,    1,  0}, // 1
		{PUSH | ARG_SRC_STR,    2,  0}, // 2
		{PUSH | ARG_SRC_STR,    3,  0}, // 3
		// Print value
		{SYSCALL           ,    3,  0}, // 4

		// Jump
		{JMP               ,   16,  0}, // 5
		{HLT               ,    0,  0}, // 6
		{HLT               ,    0,  0}, // 7
		{HLT               ,    0,  0}, // 8
		{HLT               ,    0,  0}, // 9
		{HLT               ,    0,  0}, // 10
		{PUSH | ARG_SRC_CR ,    1,  0}, // 11
		// Push INDIRECT INTEGER from register
		{PUSH | ARG_SRC_IND_VAL | ARG_DST_CR,    1,  0}, // 12
		{PUSH | ARG_SRC_IND_STR | ARG_DST_BR,    4,  0}, // 13
		{POP               ,    3,  0}, // 14
		{HLT               ,    0,  0}, // 15
		{JMP               ,    7,  0}, // 16

		// Subroutine
		//
		// function HelloWorld()
		// {
		//    print "Hello, World"; // syscall to __ctpp2_output_text
		// }

		// COMMAND              ARG RESERVERD
		// Push string into stack from static data segment striung ID 0
		{PUSH |ARG_SRC_INT ,   0x00000000,  0},                                       // 17
		// Syscall "__ctpp2_output_text", one argument
		{SYSCALL           ,   0x00010001,  0},                                       // 18
		// Clear stack
		{RET               ,    0,  0},                                               // 19
		// Return from subroutine
		{HLT               ,    0,  0}                                                // 20
		// All done
	};

	FILE * F = fopen(argv[1], "w");
	if (F == NULL) { fprintf(stderr, "ERROR: Cannot open file `%s` for writing\n", argv[1]); return EX_SOFTWARE; }

	VMDumper oDumper(21, aCode, oSyscalls, oStaticData, oStaticText);

	UINT_32 iSize = 0;
	const VMExecutable * aProgramCore = oDumper.GetExecutable(iSize);

	fwrite(aProgramCore, iSize, 1, F);

	fclose(F);

	// make valgrind happy
	fclose(stdin);
	fclose(stdout);
	fclose(stderr);

return EX_OK;
}
// End.

==> ctpp2-2.3.0.tar.gz: ctpp2-2.3.0/tests/VMHelloWorldGenerator.cpp <==
/*-
 * Copyright (c) 2006, 2007 FTS Team
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 4. Neither the name of the FTS Team nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      VMHelloWorldGenerator.cpp
 *
 * $FTS$
 */
#include <VMDumper.hpp>
#include <VMExecutable.hpp>
#include <VMOpcodes.h>
#include <VMInstruction.hpp>

#include <stdio.h>
#include <sysexits.h>

using namespace CTPP;

int main(int argc, char ** argv)
{
	if (argc != 2) { fprintf(stderr, "usage: %s file.name\n", argv[0]); return EX_USAGE; }

	// 1. Create static text segment
	StaticText oStaticText;

	StaticText oSyscalls;

	// Syscalls
	//                     12345678901234
	oSyscalls.StoreData("__ctpp2_output",          14); // returns '0'
	//                     1234567890123456789
	oSyscalls.StoreData("__ctpp2_output_text",     19); // returns '1'

	// Text Data
	//                     1234567890123456789
	oStaticText.StoreData("CTPP2 Hello, World\n",    19); // returns '0'
	//                     1234567890123
	oStaticText.StoreData("Concatenation",           13); // returns '1'
	//                     1
	oStaticText.StoreData(" ",                       1);  // returns '2'
	//                     1234
	oStaticText.StoreData("test.\n",                 6);  // returns '3'
	//                     1234
	oStaticText.StoreData("key",                     3);  // returns '4'

	// Numberic Data
	StaticData oStaticData;
	oStaticData.StoreInt(0);
	oStaticData.StoreInt(1977);
	oStaticData.StoreInt(10);
	oStaticData.StoreInt(8);
	oStaticData.StoreInt(2);
	oStaticData.StoreInt(6);
	oStaticData.StoreInt(5);
	oStaticData.StoreInt(3);

	// Code
	VMInstruction aCode[] =
	{
		// COMMAND              ARG RESERVERD
		// Call procedure on offset 17 from start of program
		{CALL              ,    25, 0},                  // 0

		// Move integer value to top of stack
		{PUSH | ARG_SRC_STR,    1,  0},                  // 1
		{PUSH | ARG_SRC_STR,    2,  0},                  // 2
		{PUSH | ARG_SRC_STR,    3,  0},                  // 3
		// Print value
		{SYSCALL           ,    3,  0},                  // 4

		// Jump
		{PUSH | ARG_SRC_INT,    2,  0},                  // 5
		{PUSH | ARG_SRC_INT,    3,  0},                  // 6
		{ADD  | ARG_DST_STACK | ARG_SRC_STACK              ,    0,  0},                  // 7

		{PUSH | ARG_SRC_INT,    4,  0},                  // 8
		{MUL  | ARG_DST_STACK | ARG_SRC_STACK             ,    0,  0},                  // 9

		{PUSH | ARG_SRC_INT,    4,  0},                  // 10
		{DIV  | ARG_DST_STACK | ARG_SRC_STACK             ,    0,  0},                  // 11

		{PUSH | ARG_SRC_INT,    5,  0},                  // 12
		{SUB  | ARG_DST_STACK | ARG_SRC_STACK             ,    0,  0},                  // 13

		{PUSH | ARG_SRC_INT,    6,  0},                  // 14
		{MOD  | ARG_DST_STACK | ARG_SRC_STACK             ,    0,  0},                  // 15

		{PUSH | ARG_SRC_INT,    7,  0},                  // 16
		{IDIV | ARG_DST_STACK | ARG_SRC_STACK             ,    0,  0},                  // 17

		{PUSH | ARG_SRC_INT,    1,  0},                  // 18
		{CMP  | ARG_DST_STACK | ARG_SRC_STACK             ,    0,  0},                  // 19
		{JE                ,   22,  0},                  // 20
		{NOP               ,    0,  0},                  // 21
		{SYSCALL           ,    1,  0},                  // 22
		{HLT               ,    0,  0},                  // 23
		{JMP               ,    6,  0},                  // 24

		// Subroutine
		//
		// function HelloWorld()
		// {
		//    print "Hello, World"; // syscall to __ctpp2_output_text
		// }

		// COMMAND              ARG RESERVERD
		// Push string into stack from static data segment string ID 0
		{PUSH |ARG_SRC_INT ,   0x00000000,  0},                                       // 25
		// Syscall "__ctpp2_output_text", one argument
		{SYSCALL           ,   0x00010001,  0},                                       // 26
		// Clear stack
		// Return from subroutine
		{RET               ,    0,  0},                                               // 27
		// Return from subroutine
		{HLT               ,    0,  0}                                                // 28
		// All done
	};

	FILE * F = fopen(argv[1], "w");
	if (F == NULL) { fprintf(stderr, "ERROR: Cannot open file `%s` for writing\n", argv[1]); return EX_SOFTWARE; }

	VMDumper oDumper(28, aCode, oSyscalls, oStaticData, oStaticText);

	UINT_32 iSize = 0;
	const VMExecutable * aProgramCore = oDumper.GetExecutable(iSize);

	fwrite(aProgramCore, iSize, 1, F);

	fclose(F);

	// make valgrind happy
	fclose(stdin);
	fclose(stdout);
	fclose(stderr);

return EX_OK;
}
// End.
